package luk;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.marshalling.MarshallerFactory;
import org.drools.marshalling.ObjectMarshallingStrategy;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;
import org.junit.Test;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class SerializationTest {

    private PoolingDataSource ds;
    private KnowledgeBase     kbase;
    private Environment       environment;
    private long              workItemId;

    @Test
    public void shouldSerializeVariable() throws Exception {
        prepareDataSource();
        Map<String, Object> configOverrides = new HashMap<String, Object>();
        configOverrides.put("hibernate.hbm2ddl.auto", "create");
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", configOverrides);
        kbase = kbase();
        environment = environment(emf);
        StatefulKnowledgeSession ksession = newSession();
        int sessionId = ksession.getId();
        Map<String, Object> parameters = new HashMap<String, Object>();
        Object object = createObject();
        parameters.put("object", object);
        ksession.insert(object);
        ksession.startProcess("process", parameters);
        // ProcessInstance process = ksession.createProcessInstance("process",
        // parameters);
        // long processId = process.getId();
        // ksession.startProcessInstance(processId);
        ksession.fireAllRules();
        ksession.dispose();
        emf.close();

        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");
        environment = environment(emf);
        ksession = loadSession(sessionId);
        WorkItemManager mgr = ksession.getWorkItemManager();
        mgr.completeWorkItem(workItemId, null);
        ksession.dispose();
        TransactionManagerServices.getTransactionManager().shutdown();
        emf.close();
        ds.close();

    }

    private Object createObject() throws InstantiationException, IllegalAccessException, Exception {
        Stuff object = new Stuff();
        object.setStatus("A");
        object.setId("A");
        return object;
    }

    private KnowledgeBase kbase() {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("process.bpmn"), ResourceType.BPMN2);
        if (kbuilder.hasErrors()) {
            KnowledgeBuilderErrors errors = kbuilder.getErrors();
            throw new RuntimeException(errors.toString());
        }
        return kbuilder.newKnowledgeBase();
    }

    private StatefulKnowledgeSession newSession() {
        KnowledgeSessionConfiguration config = config();
        StatefulKnowledgeSession session = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, config, environment);
        return prepareSession(session);
    }

    private StatefulKnowledgeSession loadSession(int sessionId) {
        KnowledgeSessionConfiguration config = config();
        StatefulKnowledgeSession session = JPAKnowledgeService.loadStatefulKnowledgeSession(sessionId, kbase, config,
                environment);
        return prepareSession(session);
    }

    private StatefulKnowledgeSession prepareSession(StatefulKnowledgeSession session) {
        WorkItemManager manager = session.getWorkItemManager();
        manager.registerWorkItemHandler("Task", new WorkItemHandler() {

            @Override
            public void executeWorkItem(WorkItem workItem, WorkItemManager manager) {
                workItemId = workItem.getId();
            }

            @Override
            public void abortWorkItem(WorkItem workItem, WorkItemManager manager) {
            }
        });
        return session;
    }

    private Environment environment(EntityManagerFactory emf) {
        Environment env = KnowledgeBaseFactory.newEnvironment();
        env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        env.set(EnvironmentName.OBJECT_MARSHALLING_STRATEGIES,
                new ObjectMarshallingStrategy[] {MarshallerFactory.newSerializeMarshallingStrategy()});
        return env;
    }

    private static KnowledgeSessionConfiguration config() {
        Properties properties = new Properties();
        properties.put("drools.processInstanceManagerFactory",
                "org.jbpm.persistence.processinstance.JPAProcessInstanceManagerFactory");
        properties.put("drools.processSignalManagerFactory",
                "org.jbpm.persistence.processinstance.JPASignalManagerFactory");
        return KnowledgeBaseFactory.newKnowledgeSessionConfiguration(properties);
    }

    private void prepareDataSource() {
        ds = new PoolingDataSource();
        ds.setUniqueName("java:comp/env/jdbc/mydatasource");
        ds.setClassName("org.h2.jdbcx.JdbcDataSource");
        ds.setMaxPoolSize(3);
        ds.setAllowLocalTransactions(true);
        ds.getDriverProperties().put("user", "sa");
        ds.getDriverProperties().put("URL", "jdbc:h2:mem:");
        ds.init();
    }
}
